@@ -61,8 +61,8 @@ ServerCapabilities <- list(
     documentOnTypeFormattingProvider = DocumentOnTypeFormattingOptions,
     # renameProvider = FALSE,
     documentLinkProvider = DocumentLinkOptions,
-    colorProvider = TRUE
-    # foldingRangeProvider = FALSE,
+    colorProvider = TRUE,
+    foldingRangeProvider = TRUE
     # selectionRangeProvider = FALSE,
     # executeCommandProvider = ExecuteCommandOptions,
     # workspace = list()
