@@ -34,6 +34,15 @@ InsertTextFormat <- list(
     Snippet = 2
 )
 
+sort_prefixes <- list(
+    arg = "0-",
+    scope = "1-",
+    workspace = "2-",
+    imported = "3-",
+    global = "4-",
+    token = "5-"
+)
+
 constants <- c("TRUE", "FALSE", "NULL",
     "NA", "NA_integer_", "NA_real_", "NA_complex_", "NA_character_",
     "Inf", "NaN")
@@ -43,8 +52,11 @@ constants <- c("TRUE", "FALSE", "NULL",
 constant_completion <- function(token) {
     consts <- constants[startsWith(constants, token)]
     completions <- lapply(consts, function(const) {
-        list(label = const, kind = CompletionItemKind$Constant,
-            data = list(type = "constant"))
+        list(label = const,
+            kind = CompletionItemKind$Constant,
+            sortText = paste0(sort_prefixes$global, const),
+            data = list(type = "constant")
+        )
     })
 }
 
@@ -54,38 +66,74 @@ package_completion <- function(token) {
     installed_packages <- .packages(all.available = TRUE)
     token_packages <- installed_packages[startsWith(installed_packages, token)]
     completions <- lapply(token_packages, function(package) {
-        list(label = package, kind = CompletionItemKind$Module,
-            data = list(type = "package"))
+        list(label = package,
+            kind = CompletionItemKind$Module,
+            sortText = paste0(sort_prefixes$global, package),
+            data = list(type = "package")
+        )
     })
     completions
 }
 
 #' Complete a function argument
 #' @keywords internal
-arg_completion <- function(workspace, token, funct, package = NULL, exported_only = TRUE) {
+arg_completion <- function(uri, workspace, point, token, funct, package = NULL, exported_only = TRUE) {
+    token_args <- NULL
+    token_data <- NULL
+
     if (is.null(package)) {
-        package <- workspace$guess_namespace(funct, isf = TRUE)
+        xdoc <- workspace$get_parse_data(uri)$xml_doc
+        if (!is.null(xdoc)) {
+            row <- point$row + 1
+            col <- point$col + 1
+            enclosing_scopes <- xdoc_find_enclosing_scopes(xdoc,
+                row, col, top = TRUE)
+            xpath <- glue(signature_xpath, row = row,
+                token_quote = xml_single_quote(funct))
+            all_defs <- xml_find_all(enclosing_scopes, xpath)
+            if (length(all_defs)) {
+                last_def <- all_defs[[length(all_defs)]]
+                func_line1 <- as.integer(xml_attr(last_def, "line1"))
+                args <- xml_text(xml_find_all(last_def, "SYMBOL_FORMALS"))
+                token_args <- args[startsWith(args, token)]
+                token_data <- list(
+                    type = "parameter",
+                    funct = funct,
+                    uri = uri,
+                    line = func_line1
+                )
+            }
+        }
+
+        if (is.null(token_args)) {
+            package <- workspace$guess_namespace(funct, isf = TRUE)
+        }
     }
+
     if (!is.null(package)) {
         args <- names(workspace$get_formals(funct, package, exported_only = exported_only))
         if (is.character(args)) {
             token_args <- args[startsWith(args, token)]
-            completions <- lapply(token_args, function(arg) {
-                list(label = arg,
-                    kind = CompletionItemKind$Variable,
-                    detail = "parameter",
-                    preselect = TRUE,
-                    insertText = paste0(arg, " = "),
-                    insertTextFormat = InsertTextFormat$PlainText,
-                    data = list(
-                        type = "parameter",
-                        funct = funct,
-                        package = package
-                ))
-            })
-            completions
+            token_data <- list(
+                type = "parameter",
+                funct = funct,
+                package = package
+            )
         }
     }
+
+    completions <- lapply(token_args, function(arg) {
+        list(label = arg,
+            kind = CompletionItemKind$Variable,
+            detail = "parameter",
+            sortText = paste0(sort_prefixes$arg, arg),
+            insertText = paste0(arg, " = "),
+            insertTextFormat = InsertTextFormat$PlainText,
+            data = token_data
+        )
+    })
+
+    completions
 }
 
 
@@ -95,14 +143,17 @@ ns_function_completion <- function(ns, token, exported_only, snippet_support) {
     functs <- functs[startsWith(functs, token)]
     if (nsname == WORKSPACE) {
         tag <- "[workspace]"
+        sort_prefix <- sort_prefixes$workspace
     } else {
         tag <- paste0("{", nsname, "}")
+        sort_prefix <- sort_prefixes$global
     }
     if (isTRUE(snippet_support)) {
         completions <- lapply(functs, function(object) {
             list(label = object,
                 kind = CompletionItemKind$Function,
                 detail = tag,
+                sortText = paste0(sort_prefix, object),
                 insertText = paste0(object, "($0)"),
                 insertTextFormat = InsertTextFormat$Snippet,
                 data = list(
@@ -115,6 +166,7 @@ ns_function_completion <- function(ns, token, exported_only, snippet_support) {
             list(label = object,
                 kind = CompletionItemKind$Function,
                 detail = tag,
+                sortText = paste0(sort_prefix, object),
                 data = list(
                     type = "function",
                     package = nsname
@@ -140,6 +192,7 @@ imported_object_completion <- function(workspace, token, snippet_support) {
                 item <- list(label = object,
                     kind = CompletionItemKind$Function,
                     detail = paste0("{", nsname, "}"),
+                    sortText = paste0(sort_prefixes$imported, object),
                     insertText = paste0(object, "($0)"),
                     insertTextFormat = InsertTextFormat$Snippet,
                     data = list(
@@ -150,6 +203,7 @@ imported_object_completion <- function(workspace, token, snippet_support) {
                 item <- list(label = object,
                     kind = CompletionItemKind$Function,
                     detail = paste0("{", nsname, "}"),
+                    sortText = paste0(sort_prefixes$imported, object),
                     data = list(
                         type = "function",
                         package = nsname
@@ -182,8 +236,10 @@ workspace_completion <- function(workspace, token,
             }
             if (nsname == WORKSPACE) {
                 tag <- "[workspace]"
+                sort_prefix <- sort_prefixes$workspace
             } else {
                 tag <- paste0("{", nsname, "}")
+                sort_prefix <- sort_prefixes$global
             }
 
             functs_completions <- ns_function_completion(ns, token,
@@ -195,6 +251,7 @@ workspace_completion <- function(workspace, token,
                 list(label = object,
                      kind = CompletionItemKind$Field,
                      detail = tag,
+                     sortText = paste0(sort_prefix, object),
                      data = list(
                          type = "nonfunction",
                          package = nsname
@@ -206,6 +263,7 @@ workspace_completion <- function(workspace, token,
                 list(label = object,
                      kind = CompletionItemKind$Field,
                      detail = tag,
+                     sortText = paste0(sort_prefix, object),
                      data = list(
                          type = "lazydata",
                          package = nsname
@@ -229,6 +287,7 @@ workspace_completion <- function(workspace, token,
                 list(label = object,
                      kind = CompletionItemKind$Field,
                      detail = tag,
+                     sortText = paste0(sort_prefixes$global, object),
                      data = list(
                          type = "nonfunction",
                          package = package
@@ -240,11 +299,10 @@ workspace_completion <- function(workspace, token,
         }
     }
 
-    imported_object <- imported_object_completion(workspace, token, snippet_support)
-
-    completions <- c(
-        completions,
-        imported_object)
+    if (is.null(package)) {
+        imported_completions <- imported_object_completion(workspace, token, snippet_support)
+        completions <- c(completions, imported_completions)
+    }
 
     completions
 }
@@ -252,15 +310,15 @@ workspace_completion <- function(workspace, token,
 scope_completion_symbols_xpath <- paste(
     "FUNCTION/following-sibling::SYMBOL_FORMALS",
     "forcond/SYMBOL",
-    "expr/LEFT_ASSIGN[not(following-sibling::expr/FUNCTION)]/preceding-sibling::expr[count(*)=1]/SYMBOL",
-    "expr/RIGHT_ASSIGN[not(preceding-sibling::expr/FUNCTION)]/following-sibling::expr[count(*)=1]/SYMBOL",
-    "equal_assign/EQ_ASSIGN[not(following-sibling::expr/FUNCTION)]/preceding-sibling::expr[count(*)=1]/SYMBOL",
+    "*/LEFT_ASSIGN[not(following-sibling::expr/FUNCTION)]/preceding-sibling::expr[count(*)=1]/SYMBOL",
+    "*/RIGHT_ASSIGN[not(preceding-sibling::expr/FUNCTION)]/following-sibling::expr[count(*)=1]/SYMBOL",
+    "*/EQ_ASSIGN[not(following-sibling::expr/FUNCTION)]/preceding-sibling::expr[count(*)=1]/SYMBOL",
     sep = "|")
 
 scope_completion_functs_xpath <- paste(
-    "expr/LEFT_ASSIGN[following-sibling::expr/FUNCTION]/preceding-sibling::expr[count(*)=1]/SYMBOL",
-    "expr/RIGHT_ASSIGN[preceding-sibling::expr/FUNCTION]/following-sibling::expr[count(*)=1]/SYMBOL",
-    "equal_assign/EQ_ASSIGN[following-sibling::expr/FUNCTION]/preceding-sibling::expr[count(*)=1]/SYMBOL",
+    "*/LEFT_ASSIGN[following-sibling::expr/FUNCTION]/preceding-sibling::expr[count(*)=1]/SYMBOL",
+    "*/RIGHT_ASSIGN[preceding-sibling::expr/FUNCTION]/following-sibling::expr[count(*)=1]/SYMBOL",
+    "*/EQ_ASSIGN[following-sibling::expr/FUNCTION]/preceding-sibling::expr[count(*)=1]/SYMBOL",
     sep = "|")
 
 scope_completion <- function(uri, workspace, token, point, snippet_support = NULL) {
@@ -272,42 +330,98 @@ scope_completion <- function(uri, workspace, token, point, snippet_support = NUL
     enclosing_scopes <- xdoc_find_enclosing_scopes(xdoc,
         point$row + 1, point$col + 1)
 
-    scope_symbols <- unique(xml_text(xml_find_all(enclosing_scopes, scope_completion_symbols_xpath)))
-    scope_symbols <- scope_symbols[startsWith(scope_symbols, token)]
-    scope_symbol_completions <- lapply(scope_symbols, function(symbol) {
+    scope_symbol_nodes <- xml_find_all(enclosing_scopes, scope_completion_symbols_xpath)
+    scope_symbol_names <- xml_text(scope_symbol_nodes)
+    scope_symbol_lines <- as.integer(xml_attr(scope_symbol_nodes, "line1"))
+    scope_symbol_selector <- startsWith(scope_symbol_names, token)
+
+    scope_symbol_names <- rev(scope_symbol_names[scope_symbol_selector])
+    scope_symbol_lines <- rev(scope_symbol_lines[scope_symbol_selector])
+    scope_symbol_selector <- !duplicated(scope_symbol_names)
+
+    scope_symbol_names <- scope_symbol_names[scope_symbol_selector]
+    scope_symbol_lines <- scope_symbol_lines[scope_symbol_selector]
+
+    scope_symbol_completions <- .mapply(function(symbol, line) {
         list(
             label = symbol,
             kind = CompletionItemKind$Field,
-            detail = "[scope]"
+            sortText = paste0(sort_prefixes$scope, symbol),
+            detail = "[scope]",
+            data = list(
+                type = "nonfunction",
+                uri = uri,
+                line = line
+            )
         )
-    })
+    }, list(scope_symbol_names, scope_symbol_lines), NULL)
+
+    scope_funct_nodes <- xml_find_all(enclosing_scopes, scope_completion_functs_xpath)
+    scope_funct_names <- xml_text(scope_funct_nodes)
+    scope_funct_lines <- as.integer(xml_attr(scope_funct_nodes, "line1"))
+    scope_funct_selector <- startsWith(scope_funct_names, token)
+
+    scope_funct_names <- rev(scope_funct_names[scope_funct_selector])
+    scope_funct_lines <- rev(scope_funct_lines[scope_funct_selector])
+    scope_funct_selector <- !duplicated(scope_funct_names)
+
+    scope_funct_names <- scope_funct_names[scope_funct_selector]
+    scope_funct_lines <- scope_funct_lines[scope_funct_selector]
 
-    scope_functs <- unique(xml_text(xml_find_all(enclosing_scopes, scope_completion_functs_xpath)))
-    scope_functs <- scope_functs[startsWith(scope_functs, token)]
     if (isTRUE(snippet_support)) {
-        scope_funct_completions <- lapply(scope_functs, function(symbol) {
+        scope_funct_completions <- .mapply(function(symbol, line) {
             list(
                 label = symbol,
                 kind = CompletionItemKind$Function,
                 detail = "[scope]",
+                sortText = paste0(sort_prefixes$scope, symbol),
                 insertText = paste0(symbol, "($0)"),
-                insertTextFormat = InsertTextFormat$Snippet
+                insertTextFormat = InsertTextFormat$Snippet,
+                data = list(
+                    type = "function",
+                    uri = uri,
+                    line = line
+                )
             )
-        })
+        }, list(scope_funct_names, scope_funct_lines), NULL)
     } else {
-        scope_funct_completions <- lapply(scope_functs, function(symbol) {
+        scope_funct_completions <- .mapply(function(symbol, line) {
             list(
                 label = symbol,
                 kind = CompletionItemKind$Function,
-                detail = "[scope]"
+                sortText = paste0(sort_prefixes$scope, symbol),
+                detail = "[scope]",
+                data = list(
+                    type = "function",
+                    uri = uri,
+                    line = line
+                )
             )
-        })
+        }, list(scope_funct_names, scope_funct_lines), NULL)
     }
 
     completions <- c(scope_symbol_completions, scope_funct_completions)
     completions
 }
 
+token_completion <- function(uri, workspace, token, exclude = NULL) {
+    xdoc <- workspace$get_parse_data(uri)$xml_doc
+    if (is.null(xdoc)) {
+        return(list())
+    }
+
+    symbols <- unique(xml_text(xml_find_all(xdoc, "//SYMBOL | //SYMBOL_SUB | //SYMBOL_FUNCTION_CALL")))
+    symbols <- symbols[startsWith(symbols, token)]
+    symbols <- setdiff(symbols, exclude)
+    token_completions <- lapply(symbols, function(symbol) {
+        list(
+            label = symbol,
+            kind = CompletionItemKind$Text,
+            sortText = paste0(sort_prefixes$token, symbol)
+        )
+    })
+}
+
 #' The response to a textDocument/completion request
 #' @keywords internal
 completion_reply <- function(id, uri, workspace, document, point, capabilities) {
@@ -322,13 +436,14 @@ completion_reply <- function(id, uri, workspace, document, point, capabilities)
     snippet_support <- isTRUE(capabilities$completionItem$snippetSupport) &&
         getOption("languageserver.snippet_support", TRUE)
 
-    completions <- list()
     token_result <- document$detect_token(point, forward = FALSE)
 
     full_token <- token_result$full_token
     token <- token_result$token
     package <- token_result$package
 
+    completions <- list()
+
     if (nzchar(full_token)) {
         if (is.null(package)) {
             completions <- c(
@@ -343,16 +458,24 @@ completion_reply <- function(id, uri, workspace, document, point, capabilities)
                 workspace, token, package, token_result$accessor == "::", snippet_support))
     }
 
-    call_result <- document$detect_call(point)
-    if (nzchar(call_result$token)) {
-        completions <- c(
-            completions,
-            arg_completion(workspace, token,
-                call_result$token, call_result$package,
-                exported_only = call_result$accessor != ":::"))
+    if (token_result$accessor == "") {
+        call_result <- document$detect_call(point)
+        if (nzchar(call_result$token)) {
+            completions <- c(
+                completions,
+                arg_completion(uri, workspace, point, token,
+                    call_result$token, call_result$package,
+                    exported_only = call_result$accessor != ":::"))
+        }
     }
 
-    logger$info("completions: ", length(completions))
+    if (is.null(token_result$package)) {
+        existing_symbols <- vapply(completions, "[[", character(1), "label")
+        completions <- c(
+            completions,
+            token_completion(uri, workspace, token, existing_symbols)
+        )
+    }
 
     Response$new(
         id,
@@ -370,10 +493,9 @@ completion_item_resolve_reply <- function(id, workspace, params) {
     if (is.null(params$data) || is.null(params$data$type)) {
     } else {
         if (params$data$type == "package") {
-            if (length(find.package(params$label))) {
+            if (length(find.package(params$label, quiet = TRUE))) {
                 desc <- utils::packageDescription(params$label, fields = c("Title", "Description"))
-                description <- paste0(trimws(
-                    strsplit(desc$Description, split = "\n")[[1]]), collapse = " ")
+                description <- gsub("\\s*\n\\s*", " ", desc$Description)
                 params$documentation <- list(
                     kind = "markdown",
                     value = sprintf("**%s**\n\n%s", desc$Title, description)
@@ -381,22 +503,45 @@ completion_item_resolve_reply <- function(id, workspace, params) {
                 resolved <- TRUE
             }
         } else if (params$data$type == "parameter") {
-            doc <- workspace$get_documentation(params$data$funct, params$data$package, isf = TRUE)
+            doc <- NULL
             doc_string <- NULL
+            if (is.null(params$data$uri)) {
+                doc <- workspace$get_documentation(params$data$funct, params$data$package, isf = TRUE)
+            } else {
+                document <- workspace$documents$get(params$data$uri)
+                func_line1 <- params$data$line
+                doc_line1 <- detect_comments(document$content, func_line1 - 1) + 1
+                if (doc_line1 < func_line1) {
+                    comment <- document$content[doc_line1:(func_line1 - 1)]
+                    doc <- convert_comment_to_documentation(comment)
+                }
+            }
             if (is.list(doc)) {
                 doc_string <- doc$arguments[[params$label]]
-            }
-            if (!is.null(doc_string)) {
-                params$documentation <- list(kind = "markdown", value = doc_string)
-                resolved <- TRUE
+                if (!is.null(doc_string)) {
+                    params$documentation <- list(kind = "markdown", value = doc_string)
+                    resolved <- TRUE
+                }
             }
         } else if (params$data$type %in% c("constant", "function", "nonfunction", "lazydata")) {
-            doc <- workspace$get_documentation(params$label, params$data$package,
-                isf = params$data$type == "function")
+            doc <- NULL
             doc_string <- NULL
+            if (is.null(params$data$uri)) {
+                doc <- workspace$get_documentation(params$label, params$data$package,
+                    isf = params$data$type == "function")
+            } else {
+                document <- workspace$documents$get(params$data$uri)
+                token_line1 <- params$data$line
+                doc_line1 <- detect_comments(document$content, token_line1 - 1) + 1
+                if (doc_line1 < token_line1) {
+                    comment <- document$content[doc_line1:(token_line1 - 1)]
+                    doc <- convert_comment_to_documentation(comment)
+                }
+            }
+
             if (is.character(doc)) {
                 doc_string <- doc
-            } else if (is.list(doc) && is.character(doc$description)) {
+            } else if (is.list(doc)) {
                 doc_string <- doc$description
             }
 
