@@ -2,9 +2,11 @@ context("Test Diagnostics")
 
 test_that("lintr works", {
     skip_on_cran()
-    client <- language_client(diagnostics = TRUE)
 
-    withr::local_tempfile(c("temp_file"), fileext = ".R")
+    dir <- tempdir()
+    client <- language_client(working_dir = dir, diagnostics = TRUE)
+
+    withr::local_tempfile(c("temp_file"), tmpdir = dir, fileext = ".R")
     writeLines("a = 1", temp_file)
 
     client %>% did_open(temp_file)
@@ -15,7 +17,6 @@ test_that("lintr works", {
     expect_equal(data$diagnostics[[1]]$message, "Use <-, not =, for assignment.")
 })
 
-
 test_that("lintr is disabled", {
     skip_on_cran()
     client <- language_client(diagnostics = FALSE)
